@@ -5,21 +5,21 @@ import urllib3
 
 from indico.config import IndicoConfig
 from indico.http.client import HTTPClient
-from indico.client.request import HTTPRequest, RequestChain
+from indico.client.request import HTTPRequest, RequestChain, PagedRequest, GraphQLRequest
 
 
 class IndicoClient:
     """
-    The Indico GraphQL Client. 
+    The Indico GraphQL Client.
 
     IndicoClient is the primary way to interact with the Indico Platform.
-    
+
     Args:
         config= (IndicoConfig, optional): IndicoConfig object with environment configuration
-    
+
     Returns:
         IndicoConfig object
-    
+
     Raises:
         RuntimeError: If api_token_path does not exist.
     """
@@ -46,16 +46,19 @@ class IndicoClient:
             return chain.result
         return response
 
+    def get_ipa_version(self):
+        return self._http.execute_request(GraphQLRequest("query getIPAVersion {\n  ipaVersion\n}\n"))['ipaVersion']
+    
     def call(self, request: Union[HTTPRequest, RequestChain]):
         """
         Make a call to the Indico IPA Platform
 
         Args:
             request (GraphQLRequest or RequestChain): GraphQL request to send to the Indico Platform
-        
+
         Returns:
             Response appropriate to the class of the provided request parameter. Often JSON but not always.
-        
+
         Raises:
             IndicoRequestError: With errors in processing the request
         """
@@ -64,3 +67,16 @@ class IndicoClient:
             return self._handle_request_chain(request)
         elif request and isinstance(request, HTTPRequest):
             return self._http.execute_request(request)
+
+    def paginate(self, request: PagedRequest):
+        """
+        Provides a generator that continues paging through responses
+        Available with List<> Requests that offer pagination
+
+        Example:
+            for s in client.paginate(ListSubmissions()):
+                print("Submission", s)
+        """
+        while request.has_next_page:
+            r = self._http.execute_request(request)
+            yield r
